import {AfterViewInit, Component, ElementRef, OnInit, Renderer, ViewChild,} from '@angular/core';
import {ActivatedRoute} from '@angular/router';
import {OverAllService} from '../../overAll.service';
import {FormBuilder, FormGroup, Validators} from '@angular/forms';
import {NzMessageService} from 'ng-zorro-antd';
import {BasiCheckComponent} from '../../../modal/basi-check/basi-check.component';
import {NzModalService} from 'ng-zorro-antd';
import {pageSize} from '../../../app.constants';
import {WarnListComponent} from '../../../modal/warn-list/warn-list.component';
import {DatePipe} from '@angular/common';

@Component({
    selector: 'jhi-basic-detail',
    templateUrl: './basic-detail.component.html',
    styles: [
            `
            .checkTags {
                position: absolute;
                top: 12px;
                left: 233px;
            }
            .time-select{
                position: absolute;
                top: 55px;
                right: 15px;
                z-index: 999;
            }
            :host ::ng-deep .tabs-smart .ant-tabs-nav-scroll div.ant-tabs-tab:nth-child(3){
                margin-right: 150px;
            }
        `
    ]
})
export class BasicDetailComponent implements OnInit, AfterViewInit {
    @ViewChild('smartCheck') smartCheck: BasiCheckComponent;
    @ViewChild('thirdTabs') thirdTabs: ElementRef;
    @ViewChild('warnList') warnList: WarnListComponent;
    loading = false;
    hostId: string;
    size = 'default';
    hostType: string;
    chartOption1;
    responseTimeStr;
    losedStr;
    cpuUsedStr;
    usedStr;
    chartOption2;
    chartOption3;
    chartOption4;
    secondOptionLeft;
    secondOptionRight;
    dataSet: any[];
    panel1 = {active: true, name: '响应时间与丢包率', disabled: false};
    panel2 = {active: true, name: 'CPU使用率及内存使用率',disabled: false};
    panel3 = {active: true, name: '磁盘使用率', disabled: false};

    //监控点
    isBasicEdit;
    itemTypeValue;
    interfaceList: any[];
    itemTypeList: any[];

    //
    checkList: any[];
    isCheck = false;
    checkStatus = {
        warning: '0',
        normal: '0',
        disabled: '0',
        high: '0'
    };

    isKey = false;
    keyList: any[];
    keyType = '1';

    //图表
    isGrapha = false;
    selectGraphadata;
    chartOptionGrapha;
    imageSelectDate = new Date();

    pageSize = pageSize;

    //分页--监控点
    pageIndex = 1;  //当前页数
    totalNum;   //总数据长度
    applicationId;    //分类ID

    //进出口流量-分页
    pageNum = 1;
    totalNumInOut;

    //分页--告警
    warn_totalNum;
    warn_pageNum = 1;

    validateForm: FormGroup;

    constructor(private routerInfo: ActivatedRoute, private message: NzMessageService,
                private overAllSer: OverAllService, private renderer: Renderer,
                private fb: FormBuilder, private modalService: NzModalService,private datePipe:DatePipe) {
    }

    ngOnInit() {

        this.routerInfo.queryParams.subscribe(queryParams => {
            this.hostId = queryParams.hostId;
            this.hostType = queryParams.type;
        });
        this.responseTime();
        this.losed();
        this.cpused();
        this.disks();
        this.used();
        this.inOutInfo();

        //监测点
        this.getCheckList();
        this.getCheckStatus();
        this.findItemType();

        //历史告警
        const obj = {};
        this.warnList.getList(obj);
    }

    ngAfterViewInit() {
        // this.renderer.setElementStyle(this.thirdTabs.nativeElement,'margin-left','108px');
    }

    //平均响应时间
    responseTime() {
        this.overAllSer.responseTime(this.hostId).subscribe(
            (res) => {
                if (res.errCode == 10000) {
                    this.responseTimeStr = res.data.responseTime;
                    const responseTime = res.data.responseTime;
                    const other = 100 - res.data.responseTime;
                    const data = [
                        {value: responseTime},
                        {value: other},
                        {
                            value: 100, name: '', tooltip: {
                                formatter: function(a) {
                                    return '';
                                }
                            }
                        }
                    ];
                    this.chartOption1 = {
                        title: {
                            text: '平均响应时间',
                            top: '5%',
                            left: '35%',
                            textStyle: {
                                color: '#999',
                                fontSize: 12,
                            }
                        },
                        tooltip: {
                            trigger: 'item',
                            show: false,
                            formatter: '{a} <br/>{b} : {c} ({d}%)'
                        },
                        color: ['#7eb6ea', '#eeeeee', 'transparent'],
                        series: [
                            {
                                type: 'pie',
                                startAngle: 180,
                                radius: ['50%', '85%'],
                                center: ['50%', '75%'],
                                hoverAnimation: true,
                                data: data,
                                itemStyle: {
                                    normal: {
                                        show: false,
                                        position: 'center'
                                    },
                                    emphasis: {
                                        shadowBlur: 10,
                                        shadowOffsetX: 0,
                                        shadowColor: 'rgba(0, 0, 0, 0.5)'
                                    }
                                },
                                labelLine: {
                                    normal: {
                                        show: false
                                    }
                                },
                            }
                        ]
                    };
                } else {
                    this.message.info(res.errMsg);
                }

            }
        );
    }

    //丢包率
    losed() {
        this.overAllSer.losed(this.hostId).subscribe(
            (res) => {
                if (res.errCode == 10000) {
                    this.losedStr = res.data.losed;
                    const losed = res.data.losed;
                    const other = 100 - res.data.losed;
                    const data = [
                        {value: losed},
                        {value: other},
                        {
                            value: 100, name: '', tooltip: {
                                formatter: function(a) {
                                    return '';
                                }
                            }
                        }
                    ];
                    this.chartOption2 = {
                        title: {
                            text: '丢包率',
                            top: '5%',
                            left: '38%',
                            textStyle: {
                                color: '#999',
                                fontSize: 12,
                            }
                        },
                        tooltip: {
                            trigger: 'item',
                            show: false,
                            formatter: '{a} <br/>{b} : {c} ({d}%)'
                        },
                        color: ['#7eb6ea', '#eeeeee', 'transparent'],
                        series: [
                            {
                                type: 'pie',
                                startAngle: 180,
                                radius: ['50%', '85%'],
                                center: ['50%', '75%'],
                                hoverAnimation: true,
                                data: data,
                                itemStyle: {
                                    normal: {
                                        show: false,
                                        position: 'center'
                                    },
                                    emphasis: {
                                        shadowBlur: 10,
                                        shadowOffsetX: 0,
                                        shadowColor: 'rgba(0, 0, 0, 0.5)'
                                    }
                                },
                                labelLine: {
                                    normal: {
                                        show: false
                                    }
                                },
                            }
                        ]
                    };
                } else {
                    this.message.info(res.errMsg);
                }

            }
        );
    }

    //cup使用率
    cpused() {
        const data = {
            hostId: this.hostId,
            hostType: this.hostType,
        };
        this.overAllSer.cupUsed(data).subscribe(
            (res) => {
                if (res.errCode == 10000) {
                    this.cpuUsedStr = res.data.cpuUsed;
                    const losed = res.data.cpuUsed;
                    const other = 100 - res.data.cpuUsed;
                    const data = [
                        {value: losed},
                        {value: other},
                        {
                            value: 100, name: '', tooltip: {
                                formatter: function(a) {
                                    return '';
                                }
                            }
                        }
                    ];
                    this.chartOption3 = {
                        title: {
                            text: 'CPU使用率',
                            top: '5%',
                            left: '38%',
                            textStyle: {
                                color: '#999',
                                fontSize: 12,
                            }
                        },
                        tooltip: {
                            trigger: 'item',
                            show: false,
                            formatter: '{a} <br/>{b} : {c} ({d}%)'
                        },
                        color: ['#7eb6ea', '#eeeeee', 'transparent'],
                        series: [
                            {
                                type: 'pie',
                                startAngle: 180,
                                radius: ['50%', '85%'],
                                center: ['50%', '75%'],
                                hoverAnimation: true,
                                data: data,
                                itemStyle: {
                                    normal: {
                                        show: false,
                                        position: 'center'
                                    },
                                    emphasis: {
                                        shadowBlur: 10,
                                        shadowOffsetX: 0,
                                        shadowColor: 'rgba(0, 0, 0, 0.5)'
                                    }
                                },
                                labelLine: {
                                    normal: {
                                        show: false
                                    }
                                },
                            }
                        ]
                    };
                } else {
                    this.message.info(res.errMsg);
                }

            }
        );
    }

    //内存使用率
    used() {
        const data = {
            hostId: this.hostId,
            hostType: this.hostType,
        };
        this.overAllSer.used(data).subscribe(
            (res) => {
                if (res.errCode == 10000) {
                    this.usedStr = res.data.used;
                    const used = res.data.used;
                    const other = 100 - res.data.used;
                    const data = [
                        {value: used},
                        {value: other},
                        {
                            value: 100, name: '', tooltip: {
                                formatter: function(a) {
                                    return '';
                                }
                            }
                        }
                    ];
                    this.chartOption4 = {
                        title: {
                            text: '内存使用率',
                            top: '5%',
                            left: '38%',
                            textStyle: {
                                color: '#999',
                                fontSize: 12,
                            }
                        },
                        tooltip: {
                            trigger: 'item',
                            show: false,
                            formatter: '{a} <br/>{b} : {c} ({d}%)'
                        },
                        color: ['#7eb6ea', '#eeeeee', 'transparent'],
                        series: [
                            {
                                type: 'pie',
                                startAngle: 180,
                                radius: ['50%', '85%'],
                                center: ['50%', '75%'],
                                hoverAnimation: true,
                                data: data,
                                itemStyle: {
                                    normal: {
                                        show: false,
                                        position: 'center'
                                    },
                                    emphasis: {
                                        shadowBlur: 10,
                                        shadowOffsetX: 0,
                                        shadowColor: 'rgba(0, 0, 0, 0.5)'
                                    }
                                },
                                labelLine: {
                                    normal: {
                                        show: false
                                    }
                                },
                            }
                        ]
                    };
                } else {
                    this.message.info(res.errMsg);
                }

            }
        );
    }

    //磁盘使用率
    disks() {
        const data = {
            hostId: this.hostId,
            hostType: this.hostType,
        };
        this.overAllSer.disks(data).subscribe(
            (res) => {
                if (res.errCode == 10000) {
                    this.secondOptionLeft = {
                        tooltip: {
                            trigger: 'axis',
                        },
                        xAxis: {
                            type: 'value',
                            show: false,
                        },
                        yAxis: {
                            type: 'category',
                            axisTick: {
                                show: false
                            },
                            axisLine: {
                                show: false,
                            },
                            // 分割线设置
                            splitLine: {
                                show: true,  //显示分割线
                            },
                            data: res.data.disks.map((item) => {
                                const line = item.name.split('/');
                                return '/' + line[line.length - 1];
                            })
                        },
                        color: ['#ed7a7b', '#f2f2f2'],
                        series: [
                            {
                                name: '已使用',
                                type: 'bar',
                                stack: '总空间',
                                label: {
                                    normal: {
                                        show: true,
                                        position: 'insideRight',
                                        formatter: (obj) => {
                                            const item = (res.data.disks[obj.dataIndex].used / (1024 * 1024 * 1024)).toFixed(2);
                                            return item + 'G';
                                        }
                                    }
                                },
                                itemStyle: {
                                    normal: {
                                        color: '#3194e0'
                                    },
                                },
                                data: res.data.disks.map((item) => {
                                    const pre = ( (item.used / item.total) * 100 ).toFixed(2);  //已使用百分比
                                    return pre;
                                })
                            },
                            {
                                name: '剩余空间',
                                type: 'bar',
                                stack: '总空间',
                                label: {
                                    normal: {
                                        show: true,
                                        position: 'insideRight',
                                        formatter: (obj) => {
                                            const cut = (res.data.disks[obj.dataIndex].total - res.data.disks[obj.dataIndex].used);
                                            const item = (cut / (1024 * 1024 * 1024)).toFixed(2);
                                            return item + 'G';
                                        }
                                    }
                                },
                                itemStyle: {
                                    normal: {
                                        color: '#bfbfbf'
                                    },
                                },
                                data: res.data.disks.map((item) => {
                                    const pre = ( (item.total - item.used) / (item.total) * 100 ).toFixed(2); //剩余百分比
                                    return pre;
                                })
                            },
                        ]
                    };

                    let total = res.data.disks.map((item) => {
                        return item.total;
                    });
                    total = this.add(total);

                    let used = res.data.disks.map((item) => {
                        return item.used;
                    });
                    used = this.add(used);

                    this.secondOptionRight = {
                        title: {
                            text: '磁盘总体使用占比',
                            x: 'center'
                        },
                        tooltip: {
                            trigger: 'item',
                            formatter: '{a} <br/>{b} : {c} ({d}%)'
                        },
                        color: ['#3194e0', '#bfbfbf'],
                        series: [
                            {
                                name: '',
                                type: 'pie',
                                radius: '55%',
                                center: ['50%', '60%'],
                                data: [total, used],
                                itemStyle: {
                                    normal: {
                                        label: {
                                            position: 'inner',
                                            show: true,
                                            formatter: '{b} : {c} ({d}%)'
                                        },
                                        labelLine: {show: true}
                                    }
                                },
                                labelLine: {
                                    normal: {
                                        show: false
                                    }
                                },
                            }
                        ]
                    };
                } else {
                    this.message.info(res.errMsg);
                }

            }
        );
    }

    add(arr) {
        let total = 0;
        for (let i = 0; i < arr.length; i++) {
            total += arr[i];
        }
        return (total / 1024 / 1024 / 1024).toFixed(2);
    }

    //进出口流量-SNMP
    inOutInfo() {
        const data = {
            obj:{
                hostid:Number(this.hostId)
            },
            pageNum:this.pageNum,
            pageCount:this.pageSize
        }
        this.overAllSer.inOutInfoSnmp(data).subscribe(
            (res) => {
                if (res.errCode == 10000) {
                    this.dataSet = res.data.inOutInfo;
                }else{
                    this.message.info(res.errMsg);
                }
            }
        );
    }

    //添加监测点
    showCheckModal() {
        this.smartCheck.showCheckModal(this.hostId);
    }

    handleCheckCancel() {
        this.isCheck = false;
    }

    showKeyModal() {
        this.isKey = true;
        this.keyTypeChange();
    }

    //键值类型变更
    keyTypeChange() {
        this.overAllSer.zabbixKey(this.keyType).subscribe(
            (res) => {
                this.keyList = res.data;
            }
        );
    }

    handleKeyCancel() {
        this.isKey = false;
    }

    handleKeyOk() {
        this.isKey = false;
    }

    //监测点列表
    getCheckList() {
        this.loading = true;
        const data = {
            pageNum: this.pageIndex,
            pageCount: pageSize,
            obj:{
                applicationid: this.applicationId,
                hostid: this.hostId,
            }
        };
        this.overAllSer.findCheckByType(data).subscribe(
            (res) => {
                this.checkList = res.data.data;
                this.totalNum = res.data.totalNum;
                this.loading = false;
            }
        );
    }

    //分页
    change(e) {
        this.pageIndex = e;
        this.getCheckList();
    }

    changeType(type) {
        this.pageIndex = 1;
        this.applicationId = type;
        this.getCheckList();
    }

    //监测点状态
    getCheckStatus() {
        this.overAllSer.findItemCount(this.hostId).subscribe(
            (res) => {
                if(res.errCode == 10000){
                    this.checkStatus = res.data;
                }
            }
        );
    }

    //删除监测点
    deleteCheckItem(item) {
        this.modalService.confirm({
            nzTitle: '删除',
            nzContent: '<b style="color: red;">确认删除该监测点吗？</b>',
            nzOkText: '确定',
            nzOkType: 'danger',
            nzOnOk: () => {
                const data = {
                    params:[item.itemid]
                };
                this.overAllSer.deleteItem(data).subscribe(
                    (res) => {
                        if (res.errCode == 10000) {
                            this.message.info('删除成功');
                            this.getCheckList();
                        } else {
                            this.message.info(res.errMsg);
                        }
                    }
                );
            },
            nzCancelText: '取消',
            nzOnCancel: () => console.log('Cancel')
        });
    }

    //图表
    showImageModal(data) {
        this.isGrapha = true;
        this.selectGraphadata = data;
        this.getGraphDate();
    }

    //获取图表数据
    getGraphDate(){
        const data = {
            date: this.datePipe.transform(this.imageSelectDate,'yyyy-MM-dd'),
            itemid: this.selectGraphadata.itemid
        };
        this.overAllSer.findGraphData(data).subscribe(
            (res) => {
                if (res.errCode == 10000 && res.data.length > 0) {
                    const result = res.data;
                    this.chartOptionGrapha = {
                        title: {
                            text: this.selectGraphadata.name
                        },
                        tooltip: {
                            trigger: 'axis'
                        },
                        xAxis: {
                            data: result.map((item) => {
                                return item.time;
                            })
                        },
                        yAxis: {
                            splitLine: {
                                show: false
                            }
                        },
                        dataZoom: [{
                            startValue: '2014-06-01'
                        }, {
                            type: 'inside'
                        }],
                        series: {
                            type: 'line',
                            data: result.map((item) => {
                                return item.value;
                            }),

                        }
                    };
                }
            }
        );
    }

    handleImageCancel() {
        this.isGrapha = false;
    }

    //监控项分类
    findItemType() {
        const data = {
            'hostids': []
        };
        data.hostids.push(this.hostId);
        this.overAllSer.findItemType(data).subscribe(
            (res) => {
                this.itemTypeList = res.data;
            }
        );
    }

    //停止/启用监控项  0  启用   1停止
    updateItem(itemid, status) {
        const data = {
            'status': status,
            'itemid': itemid
        };
        this.modalService.confirm({
            nzTitle: '',
            nzContent: '<b style="color: red;">确认修改监控项状态吗？</b>',
            nzOkText: '确定',
            nzOkType: 'danger',
            nzOnOk: () => {
                this.overAllSer.itemUpdate(data).subscribe(
                    (res) => {
                        if (res.errCode == 10000) {
                            this.getCheckList();
                        }
                        this.message.info(res.errMsg);
                    }
                );
            },
            nzCancelText: '取消',
            nzOnCancel: () => console.log('Cancel')
        });

    }
}
